#include "xparameters.h"
#include "stdio.h"
#include "xil_macroback.h"
#include "shell.h"
#include "npi.h"
//#include "xgpio.h"
#ifdef XPAR_MICROBLAZE_ID
#include "xutil.h"
#include "xil_cache.h"
#include "xbasic_types.h"
#else
#include "xscugic.h"
#endif
#include "xil_exception.h"

#include <stdio.h>
#include "platform.h"

#define SIM_

int help();
int mem_write(int, char **);
int mem_read(int, char **);
int led_write(int, char **);
int last_dump_add;
void all(void);
void axi_npi_rd_0(void);
void axi_npi_wr_0(void);
void npi_wr(int, char **);
void npi_rd(int, char **);
void npi_status_0(void);
void npi_status_rx(void);
void npi_status_tx(void);
void test_init(void);
void refill(int, char **);
int npi_status(int);
void reset_rx_npi(void);
void reset_tx_npi(void);
void reset_all_npi(void);

#ifndef XPAR_MICROBLAZE_CORE_CLOCK_FREQ_HZ
//////////// interrupt
XScuGic IntC; /* Instance of the Interrupt Controller */
XScuGic_Config *GicConig;
#endif

unsigned int str_to_uint32(char *, char **, int);
void npi_cmd(int, char **);

typedef struct {
	char *CmdStr;
	int (*func)(int argc, char **argv);
} TCommand;

TCommand Cmds[] = { { "HELP", help }, { "?", help }, { "MWR", mem_write }, {
		"A", all }, { "RF", refill }, { "W", npi_wr }, { "R", npi_rd }, {
		"MRD", mem_read }, { "RD0", axi_npi_rd_0 }, { "WR0", axi_npi_wr_0 }, {
		"S0", npi_status_0 }, { "SR", npi_status_rx }, { "ST", npi_status_tx },
		{ "RR", reset_rx_npi }, { "RT", reset_tx_npi },
		{ "RA", reset_all_npi }, { "I", test_init }, { "CMD", npi_cmd }, {
				NULL, NULL } };

void reset_all_npi(void) {
	reset_rx_npi();
	reset_tx_npi();
}
void reset_rx_npi(void) {
	axi_npi_reset(XPAR_RX_NPI_BASEADDR);
	axi_npi_fifo_reset(XPAR_RX_NPI_BASEADDR);
}

void reset_tx_npi(void) {
	axi_npi_reset(XPAR_TX_NPI_BASEADDR);
	axi_npi_fifo_reset(XPAR_TX_NPI_BASEADDR);
}

int help() {
	xil_printf("help me");
#if 0
	xil_printf("MWR----mem_write");
	xil_printf("A -----all");
	xil_printf("W, npi_wr");
	xil_printf("R, npi_rd");
	"MRD  , mem_read");
	"RD0  , axi_npi_rd_0")
	"WR0  , axi_npi_wr_0       },
	"S0, npi_status_0},
	"SR, npi_status_rx},
	"ST, npi_status_tx},
	"RR, reset_rx_npi},
	"RT, reset_tx_npi},
	"RA, reset_all_npi},
	"I, test_init},
	"CMD, npi_cmd},
#endif
	return 0;
}

//XGpio Gpio_LED;

int led_write(int argc, char **argv) {

#if 0
	int s;

	XGpio_SetDataDirection(&Gpio, 1, 0);

	s = str_to_uint32(argv[1], NULL, 0);
	XGpio_DiscreteWrite(&Gpio, 1, s);

#endif
	return 0;
}

int mem_write(int argc, char **argv) {
	unsigned int data;
	void *add;

	if (argc < 3) {
		xil_printf("Argument Count Error!\n");
		return -1;
	}

	if (*argv[3] == 'w' || *argv[3] == 'W') {
		add = (int *) str_to_uint32(argv[1], NULL, 0);
		data = str_to_uint32(argv[2], NULL, 0);
		*(int *) add = data;
	} else if (*argv[3] == 'b' || *argv[3] == 'B') {
		add = (char *) str_to_uint32(argv[1], NULL, 0);
		data = (char) str_to_uint32(argv[2], NULL, 0);
		*(char *) add = (char) data;
	} else {
		add = (short int*) str_to_uint32(argv[1], NULL, 0);
		data = (short int) str_to_uint32(argv[2], NULL, 0);
		*(short int*) add = (short int) data;
	}

	return 0;
}

void dump_memory(int address, int size) {
	volatile unsigned short int *add, data[8];
	volatile unsigned int i, j;
	volatile char ch;

	//Xil_DCacheInvalidate();
	add = (unsigned short int *) address;
	for (i = 0; i < size; i += 4) {
		xil_printf("%08X: ", add); // display address
		for (j = 0; j < 8; j++) {
			data[j] = *add;
			xil_printf("%04x", data[j]); // display data
			xil_printf(" "); // space
			add++;
		}
		xil_printf("  ");

		for (j = 0; j < 16; j++) { // display char
			ch = (data[j] & 0xff00) >> 8;
			if (ch < ' ' || ch > '~')
				xil_printf(".");
			else
				xil_printf("%c", ch);

			ch = data[j] & 0xff;
			if (ch < ' ' || ch > '~')
				xil_printf(".");
			else
				xil_printf("%c", ch);
		}
		xil_printf("\n\r");
	}
}

int mem_read(int argc, char **argv) {
	int size;



	switch (argc) {
	case 1: // command only
		dump_memory(last_dump_add, 16);
		//xil_printf("%s", argv[0]);
		break;
	case 2: // command & only start address
		last_dump_add = str_to_uint32(argv[1], NULL, 0);
		dump_memory(last_dump_add, 16);
		//			xil_printf("%s", argv[0]);
		//			xil_printf("%s", argv[1]);
		break;
	case 3: // command, start address and count
		last_dump_add = str_to_uint32(argv[1], NULL, 0);
		size = str_to_uint32(argv[2], NULL, 0);

		if (size < 0)
			size *= -1;

		//Xil_DCacheInvalidateRange(last_dump_add, size*4);
		dump_memory(last_dump_add, size);
		break;
	default:
		xil_printf("Argument Count Error!\n");
		return -1;
	}

	return 0;
}

void npi_wr(int argc, char **argv) {
	int c, d;
	c = str_to_uint32(argv[1], NULL, 0);
	d = str_to_uint32(argv[2], NULL, 0);

	xil_printf("\n\r burst  cnt = 0x%x", c);
	xil_printf("\n\r repeat cnt = 0x%x", d);
	//	axi_npi_wr(XPAR_TX_NPI_BASEADDR, XPAR_PS7_DDR_0_S_AXI_BASEADDR + 0x100000, c, d);
	axi_npi_wr(XPAR_TX_NPI_BASEADDR, DDR_MEM_BASEADDR + 0x100000, c, d);

}

void npi_rd(int argc, char **argv) {
	int c, d;
	c = str_to_uint32(argv[1], NULL, 0);
	d = str_to_uint32(argv[2], NULL, 0);

	xil_printf("\n\r burst  cnt = 0x%x", c);
	xil_printf("\n\r repeat cnt = 0x%x", d);
	//	axi_npi_wr(XPAR_TX_NPI_BASEADDR, XPAR_PS7_DDR_0_S_AXI_BASEADDR + 0x100000, c, d);
	axi_npi_wr(XPAR_TX_NPI_BASEADDR, DDR_MEM_BASEADDR + 0x100000, c, d);

}

int tick0;

void npi_cmd(int argc, char **argv) {
	int c;

	c = str_to_uint32(argv[1], NULL, 0);

	//XGpio_DiscreteWrite(&Gpio_LED, 1, c);
}

void npi_tx_handler(void * baseaddr_p) {
	int baseaddr = (int) baseaddr_p;
	unsigned int csr;

	xil_printf("a");
}

extern char uart_buffer[30];
extern char *uart_s;
extern int uart_cnt;

void npi_status_0() {
	axi_npi_status(XPAR_RX_NPI_BASEADDR);
	axi_npi_status(XPAR_TX_NPI_BASEADDR);
}

void npi_status_rx() {
	axi_npi_status(XPAR_RX_NPI_BASEADDR);
}

void npi_status_tx() {
	axi_npi_status(XPAR_TX_NPI_BASEADDR);
}

void axi_npi_rd_0() {
	axi_npi_rd(XPAR_RX_NPI_BASEADDR, DDR_MEM_BASEADDR, 0x4, 0xffff);
}

void axi_npi_wr_0() {
	axi_npi_wr(XPAR_TX_NPI_BASEADDR, DDR_MEM_BASEADDR, 0x100, 0xffff);
}

void refill(int argc, char **argv)
{
	volatile int *add, i, base_value;
	if (argc < 2) {
		xil_printf("\n\r refill # ");
		return;
	}

	base_value = str_to_uint32(argv[1], NULL, 0);

	add = (int *) (0x10000000);
	for (i = 0; i < 0x1000000; i++)
		*add++ = (i + base_value);

	//Xil_DCacheFlush();

}

void all() {
	int *add, i;

	/*
	 xil_printf("\n\rwaiting init done of ddr....");
	 while(1)
	 if((*(int *)XPAR_GPIO_DONE_BASEADDR) & 0x1)
	 break;
	 */
	xil_printf("\n\r moving....");
	/////////////////////////////////////////////////////////////////////////
	// kc705 ref design : ddr range 0xa800_0000 ~ 0xafff_ffff
	/////////////////////////////////////////////////////////////////////////
	//add = (int *)(DDR_MEM_BASEADDR + 0x01000000);
	//for(i = 0; i < 0x10000; i++) *add++ = (i + 0x100);


	/////////////////////////////////////////////////////////////////////////
	// kc705 ref design : ddr range 0xa800_0000 ~ 0xafff_ffff
	// init count ===> 0x00010000 (4byte)
	// burst unit ===> 0x80
	// burst count ==> 0x00010000 * 4 / 0x80
	/////////////////////////////////////////////////////////////////////////
	axi_npi_wr(XPAR_TX_NPI_BASEADDR, 0x11000000, 0x10000 * 4 / 0x80, 0xffff);

	for (i = 0; i < 5000; i++); // delay

	axi_npi_rd(XPAR_RX_NPI_BASEADDR, 0x10000000, 0x10000 * 4 / 0x80, 0xffff);

	//axi_npi_reset(XPAR_TX_NPI_BASEADDR);
	//axi_npi_fifo_reset(XPAR_TX_NPI_BASEADDR);
	//axi_npi_reset(XPAR_RX_NPI_BASEADDR);
	//axi_npi_fifo_reset(XPAR_RX_NPI_BASEADDR);
}

void test_init()
{
	volatile int *add, i, temp;

	add = (int *) (DDR_MEM_BASEADDR);
	for (i = 0; i < 40; i++) {
		temp = (i + 1);
		*add++ = temp;
	}
}
#if 0
static XIntc InterruptController; /* Instance of the Interrupt Controller */

void init_interrupt()
{
	int Status;

	Status = XIntc_Initialize(&InterruptController, XPAR_AXI_INTC_0_DEVICE_ID);
	if (Status != XST_SUCCESS) {
		return XST_FAILURE;
	}

	Status = XIntc_Connect(&InterruptController, XPAR_AXI_INTC_0_TX_NPI_WR_FIFO_INTERRUPT_INTR,
			(XInterruptHandler)npi_tx_handler,
			(void *)0);
	if (Status != XST_SUCCESS) {
		return XST_FAILURE;
	}

	/*
	 * Connect a device driver handler that will be called when an interrupt
	 * for the device occurs, the device driver handler performs the
	 * specific interrupt processing for the device.
	 */
	Status = XIntc_Connect(XIntcInstancePtr, INTC_DEVICE_INT_ID,
			(XInterruptHandler)DeviceDriverHandler,
			(void *)0);
	if (Status != XST_SUCCESS) {
		return XST_FAILURE;
	}

	/*
	 * Start the interrupt controller such that interrupts are enabled for
	 * all devices that cause interrupts, specify simulation mode so that
	 * an interrupt can be caused by software rather than a real hardware
	 * interrupt.
	 */
	Status = XIntc_Start(XIntcInstancePtr, XIN_SIMULATION_MODE);
	if (Status != XST_SUCCESS) {
		return XST_FAILURE;
	}

	/*
	 * Enable the interrupt for the device and then cause (simulate) an
	 * interrupt so the handlers will be called.
	 */
	XIntc_Enable(XIntcInstancePtr, INTC_DEVICE_INT_ID);

	/*
	 * Initialize the exception table.
	 */
	Xil_ExceptionInit();

	/*
	 * Register the interrupt controller handler with the exception table.
	 */
	Xil_ExceptionRegisterHandler(XIL_EXCEPTION_ID_INT,
			(Xil_ExceptionHandler)XIntc_InterruptHandler,
			XIntcInstancePtr);

	/*
	 * Enable exceptions.
	 */
	Xil_ExceptionEnable();
	XIntc_MasterEnable(XPAR_OPB_INTC_0_BASEADDR);

	XIntc_mEnableIntr(XPAR_OPB_INTC_0_BASEADDR ,
			XPAR_SYSTEM_EXT_INT_MASK |
			XPAR_OPB_TIMER_1_INTERRUPT_MASK);

	XIntc_RegisterHandler(XPAR_OPB_INTC_0_BASEADDR,
			XPAR_OPB_INTC_0_SYSTEM_EXT_INT_INTR,
			ext1_handler,
			NULL);

	XIntc_RegisterHandler(XPAR_OPB_INTC_0_BASEADDR,
			XPAR_OPB_INTC_0_OPB_TIMER_1_INTERRUPT_INTR,
			timer_handler,
			XPAR_OPB_TIMER_1_BASEADDR);

	XTmrCtr_mSetLoadReg(XPAR_OPB_TIMER_1_BASEADDR,
			0,
			50000000);

	XTmrCtr_mSetControlStatusReg(XPAR_OPB_TIMER_1_BASEADDR,
			0,
			XTC_CSR_INT_OCCURED_MASK |
			XTC_CSR_LOAD_MASK );

	XTmrCtr_mSetControlStatusReg(XPAR_OPB_TIMER_1_BASEADDR,
			0,
			XTC_CSR_ENABLE_TMR_MASK |
			XTC_CSR_ENABLE_INT_MASK |
			XTC_CSR_AUTO_RELOAD_MASK |
			XTC_CSR_DOWN_COUNT_MASK);

	XTmrCtr_mSetControlStatusReg(XPAR_OPB_TIMER_1_BASEADDR,
			1,
			XTC_CSR_AUTO_RELOAD_MASK);

}
#endif

int simulation_main_rd() {
#ifdef SIM
	volatile int i;

	set_axi_npi_as_rd(XPAR_RX_NPI_BASEADDR);
	set_axi_npi_as_wr(XPAR_TX_NPI_BASEADDR);

	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x1;
	while(1)
	if((*(int *)XPAR_GPIO_DONE_BASEADDR) & 0x1)
	break;

	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x2;
	test_init();
	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x3;

	axi_npi_rd(XPAR_RX_NPI_BASEADDR, DDR_MEM_BASEADDR, 0x5, 0x2);
	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x5;
#endif
}

int simulation_ddr_main_wr() {
	volatile int i;
	/*
	 *(int *)XPAR_LEDS_8BITS_BASEADDR = 0x1;
	 while(1)
	 if((*(int *)XPAR_GPIO_DONE_BASEADDR) & 0x1)
	 break;
	 */

	axi_npi_wr(XPAR_TX_NPI_BASEADDR, DDR_MEM_BASEADDR, 0x2, 0x3);

	while (is_axi_npi_busy(XPAR_TX_NPI_BASEADDR) == 1)
		;

	for (i = 0; i < 300; i++)
		;

	axi_npi_rd(XPAR_RX_NPI_BASEADDR, DDR_MEM_BASEADDR, 0x5, 0x2);

	axi_npi_reset(XPAR_RX_NPI_BASEADDR);
	axi_npi_reset(XPAR_TX_NPI_BASEADDR);
	axi_npi_fifo_reset(XPAR_RX_NPI_BASEADDR);
	axi_npi_fifo_reset(XPAR_TX_NPI_BASEADDR);
}

int simulation_bram_main_rd_wr() {
#ifdef SIM
	volatile int i;
	int *add;

	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x1;
	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x2;
	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x3;

	add = (int *)XPAR_AXI_BRAM_CTRL_0_S_AXI_BASEADDR;
	for(i = 0; i < 0x20; i++)
	*add++ = (i + 0x100);
	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x4;

	axi_npi_wr(XPAR_TX_NPI_BASEADDR, XPAR_AXI_BRAM_CTRL_0_S_AXI_BASEADDR + 0x1000, 0x2, 0x1);
	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x5;

	for(i = 0; i < 50; i++); // delay
	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x6;

	axi_npi_rd(XPAR_RX_NPI_BASEADDR, XPAR_AXI_BRAM_CTRL_0_S_AXI_BASEADDR, 0x2, 0x1);
	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x7;
	for(i = 0; i < 50; i++); // delay
	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x8;

	axi_npi_reset(XPAR_TX_NPI_BASEADDR);
	axi_npi_fifo_reset(XPAR_TX_NPI_BASEADDR);
	axi_npi_reset(XPAR_RX_NPI_BASEADDR);
	axi_npi_fifo_reset(XPAR_RX_NPI_BASEADDR);
	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x9;
#endif
}

int simulation_bram_main_wr() {
#ifdef SIM
	volatile int i;

	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x1;
	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x2;
	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x3;

	axi_npi_wr(XPAR_TX_NPI_BASEADDR, XPAR_AXI_BRAM_CTRL_0_S_AXI_BASEADDR, 0x2, 0x3);

	while(is_axi_npi_busy(XPAR_TX_NPI_BASEADDR) == 1);
	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x4;
	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x5;
	axi_npi_reset(XPAR_TX_NPI_BASEADDR);
	axi_npi_fifo_reset(XPAR_TX_NPI_BASEADDR);
	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x6;
#endif
}

int simulation_bram_main_rd() {
#ifdef SIM
	volatile int i;
	int *add;

	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x2;
	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x3;

	add = (int *)XPAR_AXI_BRAM_CTRL_0_S_AXI_BASEADDR;
	for(i = 0; i < 0x10; i++)
	*add++ = (i + 0x100);
	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x4;

	axi_npi_rd(XPAR_RX_NPI_BASEADDR, XPAR_AXI_BRAM_CTRL_0_S_AXI_BASEADDR+0x1000, 0x2, 0x3);
	while(is_axi_npi_busy(XPAR_RX_NPI_BASEADDR) == 1);

	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x5;
	axi_npi_reset(XPAR_RX_NPI_BASEADDR);
	axi_npi_fifo_reset(XPAR_RX_NPI_BASEADDR);
	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x6;
#endif
}

#ifndef XPAR_MICROBLAZE_ID
XScuGic IntC; /* Instance of the Interrupt Controller */
XScuGic_Config *GicConfig;

void init_int() {
	int Status;

	GicConfig = XScuGic_LookupConfig(XPAR_PS7_SCUGIC_0_DEVICE_ID);

	Status = XScuGic_CfgInitialize(&IntC, GicConfig, GicConfig->CpuBaseAddress);
	if (Status != XST_SUCCESS) {
		xil_printf("\n\r error XScuGic_CfgInitialize");
	};

	Xil_ExceptionRegisterHandler(XIL_EXCEPTION_ID_INT,
			(Xil_ExceptionHandler) XScuGic_InterruptHandler, &IntC);

	Xil_ExceptionEnable();

	Status = XScuGic_Connect(&IntC, XPAR_FABRIC_TX_NPI_WR_FIFO_INTERRUPT_INTR,
			(Xil_ExceptionHandler) npi_tx_handler, (void *) NULL);

	if (Status != XST_SUCCESS) {
		xil_printf("\n\r error XScuGic_Connect");
	};

	XScuGic_Enable(&IntC, XPAR_FABRIC_TX_NPI_WR_FIFO_INTERRUPT_INTR);

	return XST_SUCCESS;
}
#endif

int simulation_ddr_main_rd_wr() {
#ifdef SIM
	volatile int i;
	int *add;

	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x1;
	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x2;

	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x1;
	while(1)
	if((*(int *)XPAR_GPIO_DONE_BASEADDR) & 0x1)
	break;

	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x3;

	add = (int *)DDR_MEM_BASEADDR;
	for(i = 0; i < 0x20; i++)
	*add++ = (i + 0x100);
	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x4;

	axi_npi_wr(XPAR_TX_NPI_BASEADDR, DDR_MEM_BASEADDR + 0x01000000, 0x2, 0xffff);
	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x5;

	for(i = 0; i < 50; i++); // delay
	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x6;

	axi_npi_rd(XPAR_RX_NPI_BASEADDR, DDR_MEM_BASEADDR, 0x2, 0xffff);
	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x7;
	for(i = 0; i < 50; i++); // delay
	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x8;

	axi_npi_reset(XPAR_TX_NPI_BASEADDR);
	axi_npi_fifo_reset(XPAR_TX_NPI_BASEADDR);
	axi_npi_reset(XPAR_RX_NPI_BASEADDR);
	axi_npi_fifo_reset(XPAR_RX_NPI_BASEADDR);
	*(int *)XPAR_LEDS_8BITS_BASEADDR = 0x9;
#endif
}

int main_shell() {

	int cmdlp, Status;
	int argc;
	char *argv[10];

	//Status = XGpio_Initialize(&Gpio, XPAR_AXI_GPIO_0_DEVICE_ID);
	//if (Status != XST_SUCCESS) {
	//	return XST_FAILURE;
	//}

	xil_printf("\n\rdate = %s\n\r", __DATE__);
	xil_printf("\n\r");

	uart_s = &uart_buffer[0];
	uart_cnt = 0;
	xil_printf("xilinx>");
	while (1) {
		if (uart_gets()) {
			argc = parse_args(uart_buffer, argv);
			if (argc) {
				upper_str(argv[0]);
				cmdlp = 0;
				while (Cmds[cmdlp].CmdStr) {
					if (strcmp(argv[0], Cmds[cmdlp].CmdStr) == 0) {
						xil_printf("\n\r");
						Cmds[cmdlp].func(argc, argv);
						xil_printf("\n\r");
						break;
					}
					cmdlp++;
				}
				if (Cmds[cmdlp].CmdStr == NULL)
					xil_printf("\n\rInvalid command\n\r");
			} else
				xil_printf("\n\r");
			memset(uart_buffer, 0, sizeof(uart_buffer));
			xil_printf("xilinx> ");
			uart_cnt = 0;
			uart_s = &uart_buffer[0];
		}
	}
}

int main() {
	init_platform();
	main_shell();
	//simulation_bram_main_wr();
	//simulation_bram_main_rd();
	//simulation_bram_main_rd_wr();
	//simulation_ddr_main_wr();
	//simulation_main_rd();
}
