/*
 * npi.c
 *
 *  Created on: Jan 15, 2013
 *      Author: hyukk
 */

#include "npi.h"

int axi_npi_status(int npi_base_add)
{
	volatile unsigned int *add, data;
	unsigned int npi_ver, cnt_rx_bus, cnt_tx_bus, npi_ready_sts, mst_sm_sts, mst_cmd_sts, fifo_sts;
	unsigned int mst_ip2bus_addr, c_cnt_burst, cnt_burst, c_cnt_repeat, cnt_repeat;
	unsigned int rd_fifo_wr_cnt, wr_fifo_wr_cnt, mst_cntl_rd_req, mst_cntl_wr_req;
	unsigned int mst_cntl_burst, mst_cntl_bus_lock, mst_cmd_sm_xfer_length;
	unsigned int cnt_rd_fifo_empty_s, cnt_wr_fifo_full_s;

	add = (volatile unsigned int *) npi_base_add;

	data = *add++;
	fifo_sts      = data & 0xf; data = data >> 4;	// // offset 0x0
	mst_cmd_sts   = data & 0xf; data = data >> 4;
	mst_sm_sts    = data & 0xf; data = data >> 4;
	npi_ready_sts = data & 0x3; data = data >> 2;
	cnt_tx_bus    = data & 0xf; data = data >> 4;
	cnt_rx_bus    = data & 0xf; data = data >> 4;

	data = *add++;
	mst_ip2bus_addr = data;	// 	// offset 0x4

	data = *add++;
	cnt_burst   = data & 0xfff; data = data >> 12;	// 	// offset 0x8
	c_cnt_burst = data & 0xfff; data = data >> 12;

	data = *add++;
	cnt_repeat  = data & 0xffff; data = data >> 16;	// 	// offset 0xc
	c_cnt_repeat = data & 0xffff; data = data >> 16;

	data = *add++;
	rd_fifo_wr_cnt = data & 0xff; data = data >> 8;	// 	// offset 0x10
	wr_fifo_wr_cnt = data & 0xff; data = data >> 8;

	data = *add++;	// 0ffset 0x14
	mst_cntl_rd_req = data & 0x1; data = data >> 1;
	mst_cntl_wr_req = data & 0x1; data = data >> 1;
	mst_cntl_bus_lock = data & 0x1; data = data >> 1;
	mst_cntl_burst = data & 01;

	data = *add++;	// 0ffset 0x18
	cnt_wr_fifo_full_s = data & 0xffff; data >> 16;
	cnt_rd_fifo_empty_s = data & 0xffff;

	data = *add++;	// 0ffset 0x1c
	npi_ver = data & 0xffff;

	xil_printf("\n\r %30s 0x%x", "npi ver", npi_ver);

	switch (mst_sm_sts) {
		case 1: xil_printf("\n\r %30s %s", "npi is", "CMD_IDLE"); break;
		case 2: xil_printf("\n\r %30s %s", "npi is", "CMD_RUN"); break;
		case 3: xil_printf("\n\r %30s %s", "npi is", "CMD_WAIT_FOR_DATA"); break;
		case 4: xil_printf("\n\r %30s %s", "npi is", "CMD_DONE"); break;
		case 5: xil_printf("\n\r %30s %s", "npi is", "FINISH"); break;
	   default: xil_printf("\n\r %30s %s", "unknown"); break;
	}

	xil_printf("\n\r %30s 0x%x", "rd_fifo_empty is", fifo_sts & 0x1);	fifo_sts = fifo_sts >> 1;
	xil_printf("\n\r %30s 0x%x", "rd_fifo_full is", fifo_sts & 0x1);	fifo_sts = fifo_sts >> 1;
	xil_printf("\n\r %30s 0x%x", "wr_fifo_empty is", fifo_sts & 0x1);	fifo_sts = fifo_sts >> 1;
	xil_printf("\n\r %30s 0x%x", "wr_fifo_full is", fifo_sts & 0x1);	fifo_sts = fifo_sts >> 1;

	xil_printf("\n\r %30s 0x%x", "rd_fifo_ready_s is", npi_ready_sts & 0x1);	npi_ready_sts = npi_ready_sts >> 1;
	xil_printf("\n\r %30s 0x%x", "wr_fifo_ready_s is", npi_ready_sts & 0x1);	npi_ready_sts = npi_ready_sts >> 1;

	xil_printf("\n\r %30s 0x%x", "mst_go is", mst_cmd_sts & 0x1);	mst_cmd_sts = mst_cmd_sts >> 1;
	xil_printf("\n\r %30s 0x%x", "mst_cmd_sm_set_timeout is", mst_cmd_sts & 0x1);	mst_cmd_sts = mst_cmd_sts >> 1;
	xil_printf("\n\r %30s 0x%x", "mst_cmd_sm_set_error is", mst_cmd_sts & 0x1);	mst_cmd_sts = mst_cmd_sts >> 1;
	xil_printf("\n\r %30s 0x%x", "mst_cmd_sm_set_done is", mst_cmd_sts & 0x1);	mst_cmd_sts = mst_cmd_sts >> 1;

	xil_printf("\n\r %30s 0x%x", "current access add is", mst_ip2bus_addr);
	xil_printf("\n\r %30s 0x%x", "cnt_burst is", cnt_burst);
	xil_printf("\n\r %30s 0x%x", "c_cnt_burst is", c_cnt_burst);
	xil_printf("\n\r %30s 0x%x", "cnt_repeat is", cnt_repeat);
	xil_printf("\n\r %30s 0x%x", "c_cnt_repeat is", c_cnt_repeat);
	xil_printf("\n\r %30s 0x%x", "cnt_rx is", cnt_rx_bus);
	xil_printf("\n\r %30s 0x%x", "cnt_tx is", cnt_tx_bus);
	xil_printf("\n\r %30s 0x%x", "rd_fifo_wr_cnt is", rd_fifo_wr_cnt);
	xil_printf("\n\r %30s 0x%x", "wr_fifo_wr_cnt is", wr_fifo_wr_cnt);
	xil_printf("\n\r %30s 0x%x", "mst_cntl_rd_req is", mst_cntl_rd_req);
	xil_printf("\n\r %30s 0x%x", "mst_cntl_wr_req is", mst_cntl_wr_req);
	xil_printf("\n\r %30s 0x%x", "mst_cntl_bus_lock is", mst_cntl_bus_lock);
	xil_printf("\n\r %30s 0x%x", "mst_cntl_burst is", mst_cntl_burst);
	if(npi_ver == 0x400c) {
		xil_printf("\n\r %30s 0x%x", "cnt_wr_fifo_full_s is", cnt_wr_fifo_full_s);
		xil_printf("\n\r %30s 0x%x", "cnt_rd_fifo_empty_s is", cnt_rd_fifo_empty_s);
	}
}

int axi_npi_ver(int npi_base_add)
{
	int *add;

	add = (int *) npi_base_add;
	return ((*add >> 22) & 0x3ff);
}

void set_axi_npi_as_wr(int npi_base_add)
{
	if(axi_npi_ver(npi_base_add) == 0x41)
		*(volatile char *) (npi_base_add + 0x100 + 0x8) = 0x1;

}

void set_axi_npi_as_rd(int npi_base_add)
{
	if(axi_npi_ver(npi_base_add) == 0x41)
		*(volatile char *) (npi_base_add + 0x100 + 0x8) = 0x0;
}

void axi_npi_rd(int npi_base_add, int src, int cnt_burst, int cnt_repeat)
{
#define GO_CMD 0x0a
	*(volatile char *) (npi_base_add + 0x100 + 0x0) = 0x9;
	*(volatile short int *) (npi_base_add + 0x100 + 0x2) = cnt_burst;
	*(volatile short int *) (npi_base_add + 0x100 + 0x6) = (src >> 16) & 0xffff;
	*(volatile short int *) (npi_base_add + 0x100 + 0xa) = cnt_repeat;
	*(volatile char *) (npi_base_add + 0x100 + 0xf) = (GO_CMD);
	*(volatile char *) (npi_base_add + 0x100 + 0xf) = (0x0b);
}

void axi_npi_wr(int npi_base_add, int src, int cnt_burst, int cnt_repeat)
{
#define GO_CMD 0x0a
	*(volatile char *) (npi_base_add + 0x100 + 0x0) = 0xa;
	*(volatile short int *) (npi_base_add + 0x100 + 0x2) = cnt_burst;
	*(volatile short int *) (npi_base_add + 0x100 + 0x6) = (src >> 16) & 0xffff;;
	*(volatile short int *) (npi_base_add + 0x100 + 0xa) = cnt_repeat;
	*(volatile char *) (npi_base_add + 0x100 + 0xf) = (GO_CMD);
	*(volatile char *) (npi_base_add + 0x100 + 0xf) = (0x0b);
}

int is_axi_npi_busy(int npi_base_add)
{
	int *add;
	add = (unsigned int *) npi_base_add;

	return ((*add >> 4) & 0x01);
}

void axi_npi_reset(int npi_base_add)
{
#define SM_RESET_VALUE 0x4

	*(volatile char *) (npi_base_add + 0x100 + 0x0) = 0x0;
	*(volatile int *) (npi_base_add + 0x1c) = SM_RESET_VALUE;
	*(volatile int *) (npi_base_add + 0x1c) = 0x0;
}

void axi_npi_fifo_reset(int npi_base_add)
{
#define FIFO_RESET_VALUE 0x1

	*(volatile int *) (npi_base_add + 0x1c) = FIFO_RESET_VALUE;
	*(volatile int *) (npi_base_add + 0x1c) = 0x0;
}

